==> em_celery-0.1.31-py2.py3-none-any.whl: em_celery/tools/amz_offers_update_task_sender.py <==
# -*- coding: utf-8 -*-

import os
import datetime
import time
import logging
import logging.handlers
import json
import random

import click
from kombu import Connection
from dropshipping.utils.utils import is_asin_valid
from em_tasks.spapi import Spapi
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
import dateutil
import dateutil.parser

from em_celery import logger, get_offer_service, init_db
from em_celery.tasks.spapi_update_item_offers_task import spapi_update_item_offers


@click.command('Send spapi update item offers task to worker.')
@click.option('-s', '--store_code', type=str, required=True, help='Store code.')
@click.option('-b', '--broker_url', type=str, required=True, help='Celery worker broker URL.')
@click.option('-m', '--marketplace', type=str, default='us', help='Amazon marketplace to fetch offers.')
@click.option('-t', '--ttl', type=int, default=36, help='Offer alive hours, default is 36.')
@click.option('-f', '--force', is_flag=True, help='Force to update offers.')
@click.option('-q', '--qps', type=float, help='Quantity per second (QPS) to send task.')
def send_amz_offers_update_task(store_code, broker_url, qps, marketplace='us', ttl=36, force=False):
  log_dir = os.path.join(os.path.expanduser('~'), '.em_celery', 'logs')
  if not os.path.isdir(log_dir):
    os.makedirs(log_dir)
  log_path = os.path.join(log_dir, 'amz_offers_update_task_sender.log')
  level = logging.INFO
  max_bytes = 20 * 1024 ** 2
  fh = logging.handlers.RotatingFileHandler(
    log_path, maxBytes=max_bytes, backupCount=5)
  fh.setLevel(level)
  formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
  fh.setFormatter(formatter)
  logger.addHandler(fh)

  init_db()

  data_source = None
  store = StoreUtil.get_store_by_code(store_code)
  platform = store.platfrom
  if platform == 'Shopify':
    data_source = StoreUtil
  elif platform == 'Spree':
    credential = json.loads(store.api_credential)
    data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
  else:
    logger.error('[UnsupportedPlatform] Only "Shopify" and "Spree" are supported, %s given', platform)
    return

  if not data_source:
    return

  condition = 'new'
  sender = AmzOffersUpdateTaskSender(data_source, broker_url, qps, marketplace, condition, ttl, force)
  sender.run()


class AmzOffersUpdateTaskSender():
  def __init__(self, data_source, broker_url, qps, marketplace, condition, ttl, force):
    self.data_source = data_source
    self.offer_service = get_offer_service()
    self.broker_url = broker_url
    self.marketplace = marketplace.lower()
    self.qps = qps
    self.condition = condition
    self.ttl = ttl
    self.force = force
    self.connection = Connection(broker_url)
    self.queue = 'SpapiItemOffersUpdate_{}'.format(marketplace.upper())
    self.offer_type = 'lowest_offer_listings'
    self.last_send_time = None

  def run(self):
    asins_buf = []
    batch_size = 500
    for sp in self.data_source.get_amz_products(self.marketplace):
      asin = sp.get('source_product_id', None) or sp.get('asin', None)
      if not asin:
        continue

      if not is_asin_valid(asin):
        continue

      asins_buf.append(asin)
      if len(asins_buf) < batch_size:
        continue

      self.process_products(asins_buf)
      asins_buf = []

    if asins_buf:
      self.process_products(asins_buf)
      asins_buf = []

  def process_products(self, asins):
    if self.force:
      asins_without_offer = list(asins)
    else:
      now = datetime.datetime.utcnow()
      offer_expire_time = now - datetime.timedelta(hours=self.ttl)

      offers = dict()
      result = self.offer_service.search_offers(
        self.offer_type, asins, self.marketplace, self.condition)
      while isinstance(result, dict) and 'hits' in result:
        result = result['hits']
      if isinstance(result, list):
        for offer in result:
          if not offer:
            continue

          offers[offer['_source']['asin']] = offer['_source']
      else:
        offers = result
      asins_without_offer = dict()
      for asin in asins:
        if asin not in offers or not offers[asin]:
          asins_without_offer[asin] = None
          continue

        offer = offers[asin]
        offer_time_s = offer.get('time', None)
        if not offer_time_s:
          asins_without_offer[asin] = None
          continue

        try:
          offer_time = dateutil.parser.parse(offer_time_s)

          # Offer expired
          if offer_time < offer_expire_time:
            asins_without_offer[asin] = None
            continue
        except Exception as e:
          asins_without_offer[asin] = None

        logger.debug('[ASINHasOffer] %s', asin)

      asins_without_offer = list(asins_without_offer.keys())

    random.shuffle(asins_without_offer)
    chunks = [asins_without_offer[x:x + 20] for x in range(0, len(asins_without_offer), 20)]
    for chunk in chunks:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          logger.debug("Waiting %.3fs to send next message", wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      spapi_update_item_offers.apply_async(
        args=(self.marketplace, chunk, self.condition),
        queue=self.queue,
        connection=self.connection)
      logger.debug(
        'Added spapi_update_item_offers(%s, %s, %s)',
        self.marketplace, chunk, self.condition)


if __name__ == "__main__":
  send_amz_offers_update_task()

==> em_celery-0.1.42-py2.py3-none-any.whl: em_celery/tools/amz_offers_update_task_sender.py <==
# -*- coding: utf-8 -*-

import os
import datetime
import time
import logging
import logging.handlers
import json
import random

import click
from kombu import Connection
from dropshipping.utils.utils import is_asin_valid
from em_tasks.spapi import Spapi
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
import dateutil
import dateutil.parser

from em_celery import logger, get_offer_service, init_db
from em_celery.tasks.spapi_update_item_offers_task import spapi_update_item_offers


@click.command('Send spapi update item offers task to worker.')
@click.option('-s', '--store_code', type=str, required=True, help='Store code.')
@click.option('-b', '--broker_url', type=str, required=True, help='Celery worker broker URL.')
@click.option('-m', '--marketplace', type=str, default='us', help='Amazon marketplace to fetch offers.')
@click.option('-t', '--ttl', type=int, default=36, help='Offer alive hours, default is 36.')
@click.option('-f', '--force', is_flag=True, help='Force to update offers.')
@click.option('-q', '--qps', type=float, help='Quantity per second (QPS) to send task.')
def send_amz_offers_update_task(store_code, broker_url, qps, marketplace='us', ttl=36, force=False):
  log_dir = os.path.join(os.path.expanduser('~'), '.em_celery', 'logs')
  if not os.path.isdir(log_dir):
    os.makedirs(log_dir)
  log_path = os.path.join(log_dir, 'amz_offers_update_task_sender.log')
  level = logging.INFO
  max_bytes = 20 * 1024 ** 2
  fh = logging.handlers.RotatingFileHandler(
    log_path, maxBytes=max_bytes, backupCount=5)
  fh.setLevel(level)
  formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
  fh.setFormatter(formatter)
  logger.addHandler(fh)

  init_db()

  data_source = None
  store = StoreUtil.get_store_by_code(store_code)
  platform = store.platform
  if platform == 'Shopify':
    data_source = StoreUtil
  elif platform == 'Spree':
    credential = json.loads(store.api_credential)
    data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
  else:
    logger.error('[UnsupportedPlatform] Only "Shopify" and "Spree" are supported, %s given', platform)
    return

  if not data_source:
    return

  condition = 'new'
  sender = AmzOffersUpdateTaskSender(data_source, broker_url, qps, marketplace, condition, ttl, force)
  sender.run()


class AmzOffersUpdateTaskSender():
  def __init__(self, data_source, broker_url, qps, marketplace, condition, ttl, force):
    self.data_source = data_source
    self.offer_service = get_offer_service()
    self.broker_url = broker_url
    self.marketplace = marketplace.lower()
    self.qps = qps
    self.condition = condition
    self.ttl = ttl
    self.force = force
    self.connection = Connection(broker_url)
    self.queue = 'SpapiItemOffersUpdate_{}'.format(marketplace.upper())
    self.offer_type = 'lowest_offer_listings'
    self.last_send_time = None

  def run(self):
    asins_buf = []
    batch_size = 500
    for sp in self.data_source.get_amz_products(self.marketplace):
      asin = sp.get('source_product_id', None) or sp.get('asin', None)
      if not asin:
        continue

      if not is_asin_valid(asin):
        continue

      asins_buf.append(asin)
      if len(asins_buf) < batch_size:
        continue

      self.process_products(asins_buf)
      asins_buf = []

    if asins_buf:
      self.process_products(asins_buf)
      asins_buf = []

  def process_products(self, asins):
    if self.force:
      asins_without_offer = list(asins)
    else:
      now = datetime.datetime.utcnow()
      offer_expire_time = now - datetime.timedelta(hours=self.ttl)

      offers = dict()
      result = self.offer_service.search_offers(
        self.offer_type, asins, self.marketplace, self.condition)
      while isinstance(result, dict) and 'hits' in result:
        result = result['hits']
      if isinstance(result, list):
        for offer in result:
          if not offer:
            continue

          offers[offer['_source']['asin']] = offer['_source']
      else:
        offers = result
      asins_without_offer = dict()
      for asin in asins:
        if asin not in offers or not offers[asin]:
          asins_without_offer[asin] = None
          continue

        offer = offers[asin]
        offer_time_s = offer.get('time', None)
        if not offer_time_s:
          asins_without_offer[asin] = None
          continue

        try:
          offer_time = dateutil.parser.parse(offer_time_s)

          # Offer expired
          if offer_time < offer_expire_time:
            asins_without_offer[asin] = None
            continue
        except Exception as e:
          asins_without_offer[asin] = None

        logger.debug('[ASINHasOffer] %s', asin)

      asins_without_offer = list(asins_without_offer.keys())

    random.shuffle(asins_without_offer)
    chunks = [asins_without_offer[x:x + 20] for x in range(0, len(asins_without_offer), 20)]
    for chunk in chunks:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          logger.debug("Waiting %.3fs to send next message", wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      spapi_update_item_offers.apply_async(
        args=(self.marketplace, chunk, self.condition),
        queue=self.queue,
        connection=self.connection)
      logger.debug(
        'Added spapi_update_item_offers(%s, %s, %s)',
        self.marketplace, chunk, self.condition)


if __name__ == "__main__":
  send_amz_offers_update_task()
